<?php
/**
Copyright Guillaume Bourgeois (2018)

contact@souke.fr

Ce logiciel est un programme informatique servant à aider les producteurs
à distribuer leur production en circuits courts.

Ce logiciel est régi par la licence CeCILL soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant
donné sa spécificité de logiciel libre, qui peut le rendre complexe à
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement,
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
pris connaissance de la licence CeCILL, et que vous en avez accepté les
termes.
 */

namespace domain\Distribution\Distribution\Export;

use common\helpers\CSV;
use domain\Config\Unit\UnitSolver;
use domain\Distribution\Distribution\Distribution;
use domain\Order\Order\OrderBuilder;
use domain\Order\Order\OrderRepository;
use domain\Order\Order\OrderSolver;
use domain\PointSale\PointSale\PointSaleRepository;
use domain\Producer\Producer\ProducerRepository;
use domain\Producer\Producer\ProducerSolver;
use domain\Product\Product\Product;
use domain\Product\Product\ProductRepository;
use domain\Product\Product\ProductSolver;
use domain\_\AbstractGenerator;

class DistributionReportCsvGenerator extends AbstractGenerator implements DistributionExportGeneratorInterface
{
    protected UnitSolver $unitSolver;
    protected ProducerRepository $producerRepository;
    protected ProductRepository $productRepository;
    protected ProductSolver $productSolver;
    protected OrderRepository $orderRepository;
    protected OrderSolver $orderSolver;
    protected PointSaleRepository $pointSaleRepository;
    protected OrderBuilder $orderBuilder;
    protected ProducerSolver $producerSolver;

    public function loadDependencies(): void
    {
        $this->unitSolver = $this->loadService(UnitSolver::class);
        $this->producerRepository = $this->loadService(ProducerRepository::class);
        $this->productRepository = $this->loadService(ProductRepository::class);
        $this->productSolver = $this->loadService(ProductSolver::class);
        $this->orderRepository = $this->loadService(OrderRepository::class);
        $this->orderSolver = $this->loadService(OrderSolver::class);
        $this->pointSaleRepository = $this->loadService(PointSaleRepository::class);
        $this->orderBuilder = $this->loadService(OrderBuilder::class);
        $this->producerSolver = $this->loadService(ProducerSolver::class);
    }

    public function generate(Distribution $distribution, bool $save = false)
    {
        $datas = [];
        $ordersArray = $this->orderRepository->findOrdersByDistribution($distribution);
        $productsArray = $this->productRepository->findProductsByDistribution($distribution);
        $optionCsvExportAllProducts = $this->producerSolver->getConfig('option_csv_export_all_products');
        $optionCsvExportByPiece = $this->producerSolver->getConfig('option_csv_export_by_piece');
        $pointsSaleArray = $this->pointSaleRepository->findPointSales();
        foreach ($pointsSaleArray as $pointSale) {
            $this->orderBuilder->initPointSaleOrders($pointSale, $ordersArray);
        }

        // produits en colonne
        $productsNameArray = ['', 'Commentaire'];
        $productsIndexArray = [];
        $productsHasQuantity = [];
        $cpt = 2;
        foreach ($productsArray as $product) {
            $productsHasQuantity[$product->id] = 0;
            foreach (Product::$unitsArray as $unit => $dataUnit) {
                $quantity = $this->orderSolver->getProductQuantity($product, $ordersArray, true, $unit);
                if ($quantity) {
                    $productsHasQuantity[$product->id] += $quantity;
                }
            }
            if ($productsHasQuantity[$product->id] > 0 || $optionCsvExportAllProducts) {
                $productName = $this->productSolver->getNameExport($product);

                if ($optionCsvExportByPiece) {
                    $productUnit = 'piece';
                } else {
                    $productUnit = $product->unit;
                }

                $productName .= ' (' . $this->unitSolver->strUnit($productUnit, 'wording_short', true) . ')';

                $productsNameArray[] = $productName;
                $productsIndexArray[$product->id] = $cpt++;
            }
        }
        $datas[] = $productsNameArray;

        // points de vente
        foreach ($pointsSaleArray as $pointSale) {
            if (count($pointSale->orders)) {
                // listing commandes
                $datas[] = ['> ' . $pointSale->name];
                foreach ($pointSale->orders as $order) {
                    $orderLine = [$this->orderSolver->getOrderUsername($order), $this->orderSolver->getCommentReport($order)];

                    if ($optionCsvExportByPiece) {
                        foreach ($order->productOrder as $productOrder) {
                            $orderLine[$productsIndexArray[$productOrder->id_product]] = $this->orderSolver->getProductQuantityPiecesByOrders(
                                $productOrder->product,
                                [$order]
                            );
                        }
                    } else {
                        foreach ($productsIndexArray as $idProduct => $indexProduct) {
                            $orderLine[$indexProduct] = '';
                        }

                        foreach ($order->productOrder as $productOrder) {
                            if (strlen($orderLine[$productsIndexArray[$productOrder->id_product]])) {
                                $orderLine[$productsIndexArray[$productOrder->id_product]] .= ' + ';
                            }
                            $orderLine[$productsIndexArray[$productOrder->id_product]] .= $productOrder->quantity;
                            if ($productOrder->product->unit != $productOrder->unit) {
                                $orderLine[$productsIndexArray[$productOrder->id_product]] .= $this->productSolver->strUnit(
                                    $productOrder->product,
                                    'wording_short',
                                    true
                                );
                            }
                        }
                    }

                    $datas[] = $this->lineOrderReport($orderLine, $cpt);
                }

                // total point de vente
                if ($optionCsvExportByPiece) {
                    $totalsPointSaleArray = $this->totalReportPieces(
                        'Total',
                        $pointSale->orders,
                        $productsArray,
                        $productsIndexArray
                    );
                } else {
                    $totalsPointSaleArray = $this->totalReport(
                        'Total',
                        $pointSale->orders,
                        $productsArray,
                        $productsIndexArray
                    );
                }

                $datas[] = $this->lineOrderReport($totalsPointSaleArray, $cpt);

                $datas[] = [];
            }
        }

        // global
        if ($optionCsvExportByPiece) {
            $totalsGlobalArray = $this->totalReportPieces(
                '> Totaux',
                $ordersArray,
                $productsArray,
                $productsIndexArray
            );
        } else {
            $totalsGlobalArray = $this->totalReport(
                '> Totaux',
                $ordersArray,
                $productsArray,
                $productsIndexArray
            );
        }

        $datas[] = $this->lineOrderReport($totalsGlobalArray, $cpt);

        CSV::send('Commandes_' . $distribution->date . '.csv', $datas);
    }

    private function totalReport($label, $ordersArray, $productsArray, $productsIndexArray)
    {
        $totalsPointSaleArray = [$label];
        foreach ($productsArray as $product) {
            foreach (Product::$unitsArray as $unit => $dataUnit) {
                $quantity = $this->orderSolver->getProductQuantity($product, $ordersArray, false, $unit);
                if ($quantity) {
                    $index = $productsIndexArray[$product->id];
                    if (!isset($totalsPointSaleArray[$index])) {
                        $totalsPointSaleArray[$index] = '';
                    }

                    if (strlen($totalsPointSaleArray[$index])) {
                        $totalsPointSaleArray[$index] .= ' + ';
                    }

                    $totalsPointSaleArray[$index] .= $quantity;

                    if ($product->unit != $unit) {
                        $totalsPointSaleArray[$index] .= '' . $this->unitSolver->strUnit($unit, 'wording_short', true);
                    }
                }
            }
        }
        return $totalsPointSaleArray;
    }

    private function totalReportPieces($label, $ordersArray, $productsArray, $productsIndexArray)
    {
        $totalsPointSaleArray = [$label];
        foreach ($productsArray as $product) {
            $quantity = 0;
            foreach (Product::$unitsArray as $unit => $dataUnit) {
                $quantityProduct = $this->orderSolver->getProductQuantity($product, $ordersArray, false, $unit);

                if ($unit == 'piece') {
                    $quantity += $quantityProduct;
                } else {
                    if ($product->weight > 0) {
                        $quantity += ($quantityProduct * $dataUnit['coefficient']) / $product->weight;
                    }
                }
            }

            if ($quantity) {
                $index = $productsIndexArray[$product->id];
                $totalsPointSaleArray[$index] = $quantity;
            }
        }

        return $totalsPointSaleArray;
    }

    private function lineOrderReport($orderLine, $cptMax, $showTotal = false)
    {
        $line = [];
        $cptTotal = 0;

        for ($i = 0; $i <= $cptMax; $i++) {
            if (isset($orderLine[$i]) && $orderLine[$i]) {
                $line[] = $orderLine[$i];
                if (is_numeric($orderLine[$i])) {
                    $cptTotal += $orderLine[$i];
                }
            } else {
                $line[] = '';
            }
        }

        if ($cptTotal > 0 && $showTotal) {
            $line[] = $cptTotal;
        }

        return $line;
    }
}