<?php
/**
Copyright Guillaume Bourgeois (2018)

contact@souke.fr

Ce logiciel est un programme informatique servant à aider les producteurs
à distribuer leur production en circuits courts.

Ce logiciel est régi par la licence CeCILL soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant
donné sa spécificité de logiciel libre, qui peut le rendre complexe à
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement,
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
pris connaissance de la licence CeCILL, et que vous en avez accepté les
termes.
 */

namespace domain\Producer\Producer;

use common\helpers\Souke;
use common\helpers\Password;
use common\helpers\Upload;
use common\helpers\Url;
use domain\User\User\User;
use domain\User\UserProducer\UserProducer;
use domain\User\UserProducer\UserProducerBuilder;
use domain\User\UserProducer\UserProducerRepository;
use domain\_\AbstractBuilder;

class ProducerBuilder extends AbstractBuilder
{
    protected ProducerRepository $producerRepository;
    protected UserProducerRepository $userProducerRepository;
    protected UserProducerBuilder $userProducerBuilder;
    protected ProducerSolver $producerSolver;

    public function loadDependencies(): void
    {
        $this->producerRepository = $this->loadService(ProducerRepository::class);
        $this->userProducerRepository = $this->loadService(UserProducerRepository::class);
        $this->userProducerBuilder = $this->loadService(UserProducerBuilder::class);
        $this->producerSolver = $this->loadService(ProducerSolver::class);
    }

    public function instanciateProducer(): Producer
    {
        $producer = new Producer();

        $producer->order_deadline = Producer::ORDER_DEADLINE_DEFAULT;
        $producer->order_delay = Producer::ORDER_DELAY_DEFAULT;
        $this->initSponsorshipCode($producer);

        return $producer;
    }

    public function initProducer(Producer $producer): void
    {
        $this->initProducerSlug($producer);
        $this->initProducerCode($producer);
    }

    public function initProducerSlug(Producer $producer): void
    {
        $cptSlug = 0 ;
        do {
            $slug = Url::slugify($producer->name) ;
            if($cptSlug) {
                $slug .= $cptSlug ;
            }
            $producer->slug = $slug ;
            $cptSlug ++ ;
        } while($this->producerRepository->findOneProducerBySlug($producer->slug)) ;
    }

    public function initProducerCode(Producer $producer): void
    {
        $producer->code = Password::generate();
    }

    /**
     * Lie un utilisateur à un producteur.
     */
    public function addUser(User $user, Producer $producer, bool $bookmark = true, bool $newsletter = true): UserProducer
    {
        $userProducer = $this->userProducerBuilder->createUserProducerIfNotExist($user, $producer, $bookmark, $newsletter);

        if (!$userProducer->getActive()) {
            $userProducer->setActive(true);
        }

        $this->update($userProducer);

        return $userProducer;
    }

    public function updateOpendistribVersion(Producer $producer): void
    {
        $versionsArray = Souke::getVersions();
        $producer->latest_version_opendistrib = reset($versionsArray);

        $this->update($producer);
    }

    public function savePrivateKeyStripe($filename, $value)
    {
        if (strlen($value) > 0) {
            $handle = fopen($filename, "w");
            fwrite($handle, $value);
            fclose($handle);
        }
    }

    public function savePrivateKeyApiStripe(Producer $producer): void
    {
        $this->savePrivateKeyStripe(
            $this->producerSolver->getFilenamePrivateKeyApiStripe($producer),
            $producer->option_stripe_private_key
        );
    }

    public function savePrivateKeyEndpointStripe(Producer $producer): void
    {
        $this->savePrivateKeyStripe(
            $this->producerSolver->getFilenamePrivateKeyEndpointStripe($producer),
            $producer->option_stripe_endpoint_secret
        );
    }

    public function savePrivateKeysStripe(Producer $producer)
    {
        $this->savePrivateKeyApiStripe($producer);
        $this->savePrivateKeyEndpointStripe($producer);

        $producer->option_stripe_private_key = '';
        $producer->option_stripe_endpoint_secret = '';

        $producer->save();
    }

    public function initOptionsDatesBeforeSave(Producer $producer): void
    {
        $this->initOptionsDatesBeforeSaveByField($producer, 'option_dashboard_date_start');
        $this->initOptionsDatesBeforeSaveByField($producer, 'option_dashboard_date_end');
        $this->initOptionsDatesBeforeSaveByField($producer, 'option_leave_period_start');
        $this->initOptionsDatesBeforeSaveByField($producer, 'option_leave_period_end');
    }

    public function initOptionsDatesDisplay(Producer $producer): void
    {
        $this->initOptionsDatesDisplayByField($producer, 'option_dashboard_date_start');
        $this->initOptionsDatesDisplayByField($producer, 'option_dashboard_date_end');
        $this->initOptionsDatesDisplayByField($producer, 'option_leave_period_start');
        $this->initOptionsDatesDisplayByField($producer, 'option_leave_period_end');
    }

    private function initOptionsDatesBeforeSaveByField(Producer $producer, string $field): void
    {
        if ($producer->$field && strlen($producer->$field)) {
            $producer->$field = date(
                'Y-m-d',
                strtotime(str_replace('/', '-', $producer->$field))
            );
        }
    }

    private function initOptionsDatesDisplayByField(Producer $producer, string $field): void
    {
        if (strlen($producer->$field)) {
            $producer->$field = date('d/m/Y', strtotime($producer->$field));
        }
    }

    public function processUploadImage(Producer $producer, string $name, string $filenameOld = null, bool $deleteImage = false)
    {
        Upload::uploadFile($producer, $name.'File', $name, $filenameOld);
        if ($deleteImage) {
            $producer->$name = '';
            $producer->save();
        }
    }

    public function initSponsorshipCode(Producer $producer): self
    {
        $producer->setSponsorshipCode(Password::generate());
        return $this;
    }
}